@@ -22,7 +22,9 @@ class ImageCache:
                      'IPTC Object Name': 'title'}
 
 
-    def __init__(self, picture_dir, db_file, geo_reverse, required_db_schema_version=1, portrait_pairs=False):
+    def __init__(self, picture_dir, db_file, geo_reverse, portrait_pairs=False):
+        # TODO these class methods will crash if Model attempts to instantiate this using a
+        # different version from the latest one - should this argument be taken out?
         self.__modified_folders = []
         self.__modified_files = []
         self.__logger = logging.getLogger("image_cache.ImageCache")
@@ -32,11 +34,13 @@ class ImageCache:
         self.__geo_reverse = geo_reverse
         self.__portrait_pairs = portrait_pairs #TODO have a function to turn this on and off?
         self.__db = self.__create_open_db(self.__db_file)
-        self.__update_schema(required_db_schema_version)
+        # NB this is where the required schema is set
+        self.__update_schema(2)
 
         self.__keep_looping = True
         self.__pause_looping = False
         self.__shutdown_completed = False
+        self.__purge_files = False
 
         t = threading.Thread(target=self.__loop)
         t.start()
@@ -62,6 +66,8 @@ class ImageCache:
         while not self.__shutdown_completed:
             time.sleep(0.05) # make function blocking to ensure staged shutdown
 
+    def purge_files(self):
+        self.__purge_files = True
 
     def update_cache(self):
         """Update the cache database with new and/or modified files
@@ -95,40 +101,43 @@ class ImageCache:
         self.__db.commit()
 
 
-    def query_cache(self, where_clause, sort_clause = 'exif_datetime ASC'):
+    def query_cache(self, where_clause, sort_clause = 'fname ASC'):
         cursor = self.__db.cursor()
         cursor.row_factory = None # we don't want the "sqlite3.Row" setting from the db here...
-
-        if not self.__portrait_pairs: # TODO SQL insertion? Does it matter in this app?
-            sql = """SELECT file_id FROM all_data WHERE {0} ORDER BY {1}
-                """.format(where_clause, sort_clause)
-            return cursor.execute(sql).fetchall()
-        else: # make two SELECTS
-            sql = """SELECT
-                        CASE
-                            WHEN is_portrait = 0 THEN file_id
-                            ELSE -1
-                        END
-                        FROM all_data WHERE {0} ORDER BY {1}
-                                    """.format(where_clause, sort_clause)
-            full_list = cursor.execute(sql).fetchall()
-            sql = """SELECT file_id FROM all_data
-                        WHERE ({0}) AND is_portrait = 1 ORDER BY {1}
-                                    """.format(where_clause, sort_clause)
-            pair_list = cursor.execute(sql).fetchall()
-            newlist = []
-            for i in range(len(full_list)):
-                if full_list[i][0] != -1:
-                    newlist.append(full_list[i])
-                elif pair_list: #OK @rec - this is tidier and qicker!
-                    elem = pair_list.pop(0)
-                    if pair_list:
-                        elem += pair_list.pop(0)
-                    newlist.append(elem)
-            return newlist
+        try:
+            if not self.__portrait_pairs: # TODO SQL insertion? Does it matter in this app?
+                sql = """SELECT file_id FROM all_data WHERE {0} ORDER BY {1}
+                    """.format(where_clause, sort_clause)
+                return cursor.execute(sql).fetchall()
+            else: # make two SELECTS
+                sql = """SELECT
+                            CASE
+                                WHEN is_portrait = 0 THEN file_id
+                                ELSE -1
+                            END
+                            FROM all_data WHERE {0} ORDER BY {1}
+                                        """.format(where_clause, sort_clause)
+                full_list = cursor.execute(sql).fetchall()
+                sql = """SELECT file_id FROM all_data
+                            WHERE ({0}) AND is_portrait = 1 ORDER BY {1}
+                                        """.format(where_clause, sort_clause)
+                pair_list = cursor.execute(sql).fetchall()
+                newlist = []
+                for i in range(len(full_list)):
+                    if full_list[i][0] != -1:
+                        newlist.append(full_list[i])
+                    elif pair_list:
+                        elem = pair_list.pop(0)
+                        if pair_list:
+                            elem += pair_list.pop(0)
+                        newlist.append(elem)
+                return newlist
+        except:
+            return []
 
 
     def get_file_info(self, file_id):
+        if not file_id: return None
         sql = "SELECT * FROM all_data where file_id = {0}".format(file_id)
         row = self.__db.execute(sql).fetchone()
         if row is not None and row['latitude'] is not None and row['longitude'] is not None and row['location'] is None:
@@ -136,6 +145,11 @@ class ImageCache:
                 row = self.__db.execute(sql).fetchone() # description inserted in table
         return row # NB if select fails (i.e. moved file) will return None
 
+    def get_column_names(self):
+        sql = "PRAGMA table_info(all_data)"
+        rows = self.__db.execute(sql).fetchall()
+        return [row['name'] for row in rows]
+
     def __get_geo_location(self, lat, lon): # TODO periodically check all lat/lon in meta with no location and try again
         location = self.__geo_reverse.get_address(lat, lon)
         if len(location) == 0:
@@ -262,11 +276,30 @@ class ImageCache:
 
         # Here, we need to update the db schema as necessary
         if schema_version < required_db_schema_version:
-            pass
-
-        # Finally, update the schema version stamp
-        self.__db.execute('DELETE FROM db_info')
-        self.__db.execute('INSERT INTO db_info VALUES(?)', (required_db_schema_version,))
+            if schema_version <= 1:
+                self.__db.execute("DROP VIEW all_data") # remake all_data for all updates
+                self.__db.execute("ALTER TABLE folder ADD COLUMN missing INTEGER DEFAULT 0 NOT NULL")
+                self.__db.execute("""
+                    CREATE VIEW IF NOT EXISTS all_data
+                    AS
+                    SELECT
+                        folder.name || "/" || file.basename || "." || file.extension AS fname,
+                        file.last_modified,
+                        meta.*,
+                        meta.height > meta.width as is_portrait,
+                        location.description as location
+                    FROM file
+                        INNER JOIN folder
+                            ON folder.folder_id = file.folder_id
+                        LEFT JOIN meta
+                            ON file.file_id = meta.file_id
+                        LEFT JOIN location
+                            ON location.latitude = meta.latitude AND location.longitude = meta.longitude
+                    WHERE folder.missing = 0
+                    """)
+            # Finally, update the schema version stamp
+            self.__db.execute('DELETE FROM db_info')
+            self.__db.execute('INSERT INTO db_info VALUES(?)', (required_db_schema_version,))
 
 
     def __get_modified_folders(self):
@@ -275,7 +308,7 @@ class ImageCache:
         for dir in [d[0] for d in os.walk(self.__picture_dir)]:
             mod_tm = int(os.stat(dir).st_mtime)
             found = self.__db.execute(sql_select, (dir,)).fetchone()
-            if not found or found['last_modified'] < mod_tm:
+            if not found or found['last_modified'] < mod_tm or found['missing'] == 1:
                 out_of_date_folders.append((dir, mod_tm))
         return out_of_date_folders
 
@@ -283,9 +316,9 @@ class ImageCache:
     def __get_modified_files(self, modified_folders):
         out_of_date_files = []
         sql_select = "SELECT fname, last_modified FROM all_data WHERE fname = ? and last_modified >= ?"
-        for dir,date in modified_folders:
+        for dir,_date in modified_folders:
             for file in os.listdir(dir):
-                base, extension = os.path.splitext(file)
+                _base, extension = os.path.splitext(file)
                 if (extension.lower() in ImageCache.EXTENSIONS
                         and not '.AppleDouble' in dir and not file.startswith('.')): # have to filter out all the Apple junk
                     full_file = os.path.join(dir, file)
@@ -298,7 +331,7 @@ class ImageCache:
 
     def __insert_file(self, file):
         file_insert = "INSERT OR REPLACE INTO file(folder_id, basename, extension, last_modified) VALUES((SELECT folder_id from folder where name = ?), ?, ?, ?)"
-        folder_insert = "INSERT OR IGNORE INTO folder(name) VALUES(?)"
+        folder_insert = "INSERT OR REPLACE INTO folder(name, missing) VALUES(?, 0)"
         mod_tm =  os.path.getmtime(file)
         dir, file_only = os.path.split(file)
         base, extension = os.path.splitext(file_only)
@@ -336,22 +369,26 @@ class ImageCache:
             if not os.path.exists(row['name']):
                 folder_id_list.append([row['folder_id']])
 
-        # Delete any non-existent folders from the db. Note, this will automatically
+        # Flag or delete any non-existent folders from the db. Note, deleting will automatically
         # remove orphaned records from the 'file' and 'meta' tables
         if len(folder_id_list):
-            self.__db.executemany('DELETE FROM folder WHERE folder_id = ?', folder_id_list)
+            if self.__purge_files:
+                self.__db.executemany('DELETE FROM folder WHERE folder_id = ?', folder_id_list)
+            else:
+                self.__db.executemany('UPDATE folder SET missing = 1 WHERE folder_id = ?', folder_id_list)
 
         # Find files in the db that are no longer on disk
-        file_id_list = []
-        for row in self.__db.execute('SELECT file_id, fname from all_data'):
-            if not os.path.exists(row['fname']):
-                file_id_list.append([row['file_id']])
-
-        # Delete any non-existent files from the db. Note, this will automatically
-        # remove matching records from the 'meta' table as well.
-        if len(file_id_list):
-            self.__db.executemany('DELETE FROM file WHERE file_id = ?', file_id_list)
+        if self.__purge_files:
+            file_id_list = []
+            for row in self.__db.execute('SELECT file_id, fname from all_data'):
+                if not os.path.exists(row['fname']):
+                    file_id_list.append([row['file_id']])
 
+            # Delete any non-existent files from the db. Note, this will automatically
+            # remove matching records from the 'meta' table as well.
+            if len(file_id_list):
+                self.__db.executemany('DELETE FROM file WHERE file_id = ?', file_id_list)
+            self.__purge_files = False
 
     def __get_exif_info(self, file_path_name):
         exifs = get_image_meta.GetImageMeta(file_path_name)
@@ -376,13 +413,19 @@ class ImageCache:
         e['focal_length'] =  exifs.get_exif('EXIF FocalLength')
         e['rating'] = exifs.get_exif('EXIF Rating')
         e['lens'] = exifs.get_exif('EXIF LensModel')
+        e['exif_datetime'] = None
         val = exifs.get_exif('EXIF DateTimeOriginal')
         if val != None:
             # Remove any subsecond portion of the DateTimeOriginal value. According to the spec, it's
             # not valid here anyway (should be in SubSecTimeOriginal), but it does exist sometimes.
             val = val.split('.', 1)[0]
-            e['exif_datetime'] = time.mktime(time.strptime(val, '%Y:%m:%d %H:%M:%S'))
-        else:
+            try:
+                e['exif_datetime'] = time.mktime(time.strptime(val, '%Y:%m:%d %H:%M:%S'))
+            except:
+                pass
+
+        # If we still don't have a date/time, just use the file's modificaiton time
+        if e['exif_datetime'] == None:
             e['exif_datetime'] = os.path.getmtime(file_path_name)
 
         gps = exifs.get_location()
